package math;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.JPanel;

import salvo.jesus.graph.Edge;
import salvo.jesus.graph.EdgeImpl;
import salvo.jesus.graph.Tree;
import salvo.jesus.graph.TreeImpl;
import salvo.jesus.graph.Vertex;
import salvo.jesus.graph.VertexImpl;
import salvo.jesus.graph.visual.GraphEditor;
import salvo.jesus.graph.visual.GraphPanel;
import salvo.jesus.graph.visual.VisualGraph;
import salvo.jesus.graph.visual.layout.GraphLayoutManager;
import salvo.jesus.graph.visual.layout.LayeredTreeLayout;

public class Main extends JFrame {

	private VisualGraph vgraph;

	private Tree buildTree() throws Exception {
		Tree tree = new TreeImpl();
		Vertex va, vb, vc, vd, vadd, vsub, vmult;
		Edge e;

		va = new VertexImpl("a");
		vb = new VertexImpl("b");
		vc = new VertexImpl("c");
		vd = new VertexImpl("d");
		vadd = new VertexImpl("+");
		vsub = new VertexImpl("-");
		vmult = new VertexImpl("*");

		tree.addNode(null, vmult);
		tree.addNode(vmult, vsub);
		tree.addNode(vmult, vadd);
		tree.addNode(vadd, vb);
		tree.addNode(vadd, va);
		tree.addNode(vsub, vd);
		tree.addNode(vsub, vc);

		return tree;
	}

	private void showTree(Tree tree) {
		GraphEditor editor;
		GraphLayoutManager layoutManager;

		editor = new GraphEditor();
		editor.setGraph(tree);
		vgraph = editor.getVisualGraph();

		layoutManager = new LayeredTreeLayout(editor.getVisualGraph());
		editor.setGraphLayoutManager(layoutManager);

		this.getContentPane().setLayout(new GridLayout(1, 2));
		this.getContentPane().add(editor);

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = new Dimension(screenSize.width - 80,
				screenSize.height - 80);

		this.setSize(frameSize);
		this.setLocation(
				(int) (screenSize.getWidth() - frameSize.getWidth()) / 2,
				(int) (screenSize.getHeight() - frameSize.getHeight()) / 2);

		this.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
	}

	public static void main(String[] args) throws Exception {
		Context context = new Context();
		context.assign("a", 20);
		context.assign("b", 40);
		context.assign("c", 30);
		context.assign("d", 10);

		TerminalExpression ta = new TerminalExpression("a");
		TerminalExpression tb = new TerminalExpression("b");
		TerminalExpression tc = new TerminalExpression("c");
		TerminalExpression td = new TerminalExpression("d");
		AddExpression add = new AddExpression(ta, tb);
		SubtractExpression sub = new SubtractExpression(tc, td);
		MultiplyExpression mult = new MultiplyExpression(add, sub);

		Calculator calculator = new Calculator(context);
		mult.accept(calculator); // ***
		System.out.println("(20+40)*(30-10)=" + calculator.getTotal());

		Main app = new Main();
		Tree tree = app.buildTree();
		app.showTree(tree);
		app.setTitle("Simple Math");
		app.setVisible(true);
		app.vgraph.layout();

	}

}
